exports = module?.exports || (window.jsonpatch = {}))

InvalidPatch = (message) ->
  Error.call this, message
  @message = message
PatchApplyError = (message) ->
  Error.call this, message
  @message = message
make_operation = (pointer, method, value) ->
  (doc) ->
    method.call pointer, doc, value
apply_patch = undefined
JSONPatch = undefined
JSONPointer = undefined
_operations = undefined
isArray = undefined
isArray = Array.isArray or (obj) ->
  Object::toString.call(obj) is "[object Array]"

exports.apply_patch = apply_patch = (doc, patch) ->
  (new JSONPatch(patch)).apply doc

exports.InvalidPatch = InvalidPatch
InvalidPatch:: = new Error()
exports.PatchApplyError = PatchApplyError
PatchApplyError:: = new Error()
exports.JSONPointer = JSONPointer = JSONPointer = (pathStr) ->
  i = undefined
  split = undefined
  path = []
  split = pathStr.split("/")
  throw new InvalidPatch("JSONPointer must start with a slash (or be an empty string)!")  if "" isnt split[0]
  i = 1
  while i < split.length
    path[i - 1] = decodeURIComponent(split[i])
    i++
  @path = path
  @length = path.length

JSONPointer::_get_segment = (index, doc) ->
  segment = @path[index]
  if isArray(doc)
    segment = parseInt(segment, 10)
    throw new PatchApplyError("Expected a number to segment an array")  if isNaN(segment) or segment < 0
  segment

JSONPointer::_action = (doc, handler) ->
  i = undefined
  segment = undefined
  i = 0
  while i < @length - 1
    segment = @_get_segment(i, doc)
    throw new PatchApplyError("Path not found in document")  unless Object.hasOwnProperty.call(doc, segment)
    doc = doc[segment]
    i++
  handler doc, @_get_segment(@length - 1, doc)

JSONPointer::add = (doc, value) ->
  if 0 is @length
    if "undefined" is typeof doc
      return value
    else
      throw new PatchApplyError("Add operation must not point to an existing value!")
  @_action doc, (node, lastSegment) ->
    if isArray(node)
      throw new PatchApplyError("Add operation must not attempt to create a sparse array!")  if lastSegment > node.length
      node.splice lastSegment, 0, value
    else
      throw new PatchApplyError("Add operation must not point to an existing value!")  if Object.hasOwnProperty.call(node, lastSegment)
      node[lastSegment] = value
    doc

JSONPointer::remove = (doc) ->
  return `undefined`  if 0 is @length
  @_action doc, (node, lastSegment) ->
    throw new PatchApplyError("Remove operation must point to an existing value!")  unless Object.hasOwnProperty.call(node, lastSegment)
    if isArray(node)
      node.splice lastSegment, 1
    else
      delete node[lastSegment]
    doc

JSONPointer::replace = (doc, value) ->
  @add @remove(doc), value

JSONPointer::get = (doc, value) ->
  @_action doc, (node, lastSegment) ->
    node[lastSegment]

_operations =
  add: JSONPointer::add
  remove: JSONPointer::remove
  replace: JSONPointer::replace

exports.JSONPatch = JSONPatch = JSONPatch = (patch) ->
  @_compile patch

JSONPatch::_compile = (patch) ->
  i = undefined
  n = undefined
  key = undefined
  op = undefined
  path = undefined
  value = undefined
  pointer = undefined
  @compiledOps = []
  patch = JSON.parse(patch)  if "string" is typeof patch
  throw new InvalidPatch("Patch must be an array of operations")  unless isArray(patch)
  i = 0
  while i < patch.length
    op = null
    for key of patch[i]
      if _operations.hasOwnProperty(key)
        throw new InvalidPatch("Only one operation allowed per block!")  if op
        op = key
        path = patch[i][op]
      else throw new InvalidPatch("Invalid operation!")  if Object.hasOwnProperty.call(patch[i], key)  if "value" isnt key
    throw new InvalidPatch("Operation missing!")  if null is op
    value = patch[i].value
    if "remove" is op and "undefined" isnt typeof (value)
      throw new InvalidPatch("\"remove\" operation should not have a \"value\"!")
    else throw new InvalidPatch("\"" + op + "\" operation should have a \"value\"!")  if "remove" isnt op and "undefined" is typeof (value)
    pointer = new JSONPointer(path)
    @compiledOps[i] = make_operation(pointer, _operations[op], value)
    i++

exports.JSONPatch::apply = (doc) ->
  i = undefined
  i = 0
  while i < @compiledOps.length
    doc = @compiledOps[i](doc)
    i++
  doc
